#include "mymodel.h"

#include <QtCore/QDebug>

#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlQueryModel>

MyModel::MyModel(QObject *parent) :
    QAbstractItemModel(parent),
    m_model(new QSqlQueryModel())
{
}

MyModel::~MyModel()
{
    delete m_model;
}

// Выбираем данные для QSqlQueryModel
bool MyModel::select()
{
    QSqlDatabase database = QSqlDatabase::database();

    if (!database.isOpen()) {
        if (!database.open()) {
            qDebug() << trUtf8("MyModel::select(): database open error: %1").arg(database.lastError().text());
            return false;
        }
    }

    QSqlQuery query(database);

    const QString queryText = QLatin1String("SELECT t01.id AS id, "
                                                   "t01.color AS color "
                                            "FROM colors t01 "
                                            "WHERE 1 = 1 "
                                            "ORDER BY t01.id");

    if (!query.prepare(queryText)) {
        qDebug() << trUtf8("MyModel::select(): query prepare error: %1").arg(query.lastError().text());
        return false;
    }

    if (!query.exec()) {
        qDebug() << trUtf8("MyModel::select(): query execution error: %1").arg(query.lastError().text());
        return false;
    }

    m_model->setQuery(query);

    return true;
}

QModelIndex MyModel::index(int row, int column, const QModelIndex &parent) const
{
    // Реализацию тупо сдираем с QAbstractTableModel
    return hasIndex(row, column, parent) ? createIndex(row, column, 0) : QModelIndex();
}

QModelIndex MyModel::parent(const QModelIndex &) const
{
    // Та же история, что и с методом MyModel::index() :-)
    return QModelIndex();
}

// Количество строк в модели
int MyModel::rowCount(const QModelIndex &parent) const
{
    // Количество строк в модели равняется количеству
    // строк в таблице в базе данных,
    // плюс одна наша пустая строка
    return m_model->rowCount(parent) + 1;
}

// Количество колонок в модели
int MyModel::columnCount(const QModelIndex &parent) const
{
    // Количество колонок в модели
    // равняется количеству колонок
    // в таблице в базе данных
    return m_model->columnCount(parent);
}

// Данные в модели
QVariant MyModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid()) {
        return QVariant();
    }

    // Если запрашиваются данные для отображения и
    // запрашиваемая строка является нулевой
    if (role == Qt::DisplayRole &&
        index.row() == 0) {
        // Смотрим индекс
        // запрашивоемой колонки
        switch (index.column()) {
        // Если это колонка с идентификатором...
        case MY_MODEL_ID_COLUMN:
            // То возвращаем -1
            return QVariant(-1);
        // А если это колонка с именем цвета...
        case MY_MODEL_COLOR_COLUMN:
            // То возвращаем текст "Цвет не выбран"
            return QVariant(trUtf8("Цвет не выбран"));
        // Во всех других случаях
        // возвращаем "Пусто"
        default:
            return QVariant();
        }
    }

    // Если запрашивается любая другая строка,
    // кроме нулевой, возвращаем данные из
    // таблицы базы данных, не забыв сделать
    // смещение на одну строку
    return m_model->data(index.sibling(index.row() - 1, index.column()), role);
}
